import numpy as np
import objective_function


class GradientDescent:
    def __init__(self, func, fprime, x_0, max_iter=1000, tol=1e-5):
        self.num_iter = 0
        self.max_iter = max_iter
        self.tol = tol
        self.f = func
        self.f_p = fprime
        self.x_k = x_0
        self.f_k = func(self.x_k)
        self.p_k = -fprime(self.x_k)

    def step(self, alpha, ):
        self.x_k = self.x_k + alpha * self.p_k
        self.f_k = self.f(self.x_k)
        self.p_k = -self.f_p(self.x_k)

    def armijo(self, stepsize):
        ALPHA = 0.5

        left = self.f(self.x_k + stepsize * self.p_k) - self.f_k
        right = -1 * ALPHA * stepsize * (self.p_k * self.p_k)

        rule = left <= right

        return rule

    def armijo_step(self):
        BETA = 0.8
        stepsize = 0.1
        while self.armijo(stepsize=stepsize) is False:
            stepsize = BETA * stepsize
        return stepsize


if __name__ == "__main__":
    x_0 = 0.5
    o_func = objective_function.ObjectiveFunction()
    gd = GradientDescent(func=o_func.f_ex2, fprime=o_func.f_prime_ex2, x_0=x_0)

    for i in range(gd.max_iter):
        step_size = gd.armijo_step()
        gd.step(alpha=step_size)
        gd.num_iter += 1

    if gd.num_iter == gd.max_iter:
        print('Result:\n  not converge.')
    elif abs(gd.p_k) < gd.tol:
        print('Result:\n  y = {:.4f}\n  x = {:.4f}'.format(gd.f_k, gd.x_k))
